<?php

/**
 * @desc 	  MoodlePayments  - Pay for a course enrolment
 * @since 	  1.8
 * @version	  1.3
 * @author	  Luuk Verhoeven [luuk@sebsoft.nl] / Sebastian Berm [sebastian@sebsoft.nl]
 * @package    enrol/mpay
 * @copyright  Sebsoft.nl
 */
defined('MOODLE_INTERNAL') || die();
require_once("$CFG->libdir/formslib.php");
require_once("$CFG->libdir/tablelib.php");
//INIT
require_once(dirname(__FILE__) . '/forms.php');
require_once(dirname(__FILE__) . '/class/mpay.class.php');
require_once(dirname(__FILE__) . '/class/currencies.php');
require_once(dirname(__FILE__) . '/basesettings.php');

//require_once $CFG->dirroot . '/enrol/authorize/localfuncs.php';

/**
 * mpay enrolment plugin implementation.
 */
if (DEVMODE == 1)
{
    ini_set('display_errors', 1);
    error_reporting(E_ALL);
}

class enrol_mpay_plugin extends enrol_plugin
{

    /**
     * Returns optional enrolment information icons.
     *
     * This is used in course list for quick overview of enrolment options.
     *
     * We are not using single instance parameter because sometimes
     * we might want to prevent icon repetition when multiple instances
     * of one type exist. One instance may also produce several icons.
     *
     * @param array $instances all enrol instances of this type in one course
     * @return array of pix_icon
     */
    private $plugin = array();
    private $client;
    private $ixr;

    function __construct()
    {
        global $DB;

        //CREATING plugin array
        $plugin_options = $DB->get_records('config_plugins', array('plugin' => 'enrol_mpay'));

        foreach ($plugin_options as $option)
        {
            $this->plugin[$option->name] = $option->value;
        }
    }

    public function get_info_icons(array $instances)
    {
        return array(new pix_icon('icon', get_string('pluginname', 'enrol_mpay'), 'enrol_mpay'));
    }

    public function roles_protected()
    {
        // users with role assign cap may tweak the roles later
        return false;
    }

    public function allow_unenrol(stdClass $instance)
    {
        // users with unenrol cap may unenrol other users manually - requires enrol/mpay:unenrol
        return true;
    }

    public function allow_manage(stdClass $instance)
    {
        // users with manage cap may tweak period and status - requires enrol/mpay:manage
        return true;
    }

    public function show_enrolme_link(stdClass $instance)
    {
        return ($instance->status == ENROL_INSTANCE_ENABLED);
    }

    /**
     * Sets up navigation entries.
     *
     * @param object $instance
     * @return void
     */
    public function add_course_navigation($instancesnode, stdClass $instance)
    {
        if ($instance->enrol !== 'mpay')
        {
            throw new coding_exception('Invalid enrol instance type!');
        }

        $context = context_course::instance($instance->courseid);
        if (has_capability('enrol/mpay:config', $context))
        {
            $managelink = new moodle_url('/enrol/mpay/edit.php', array('courseid' => $instance->courseid, 'id' => $instance->id));
            $instancesnode->add($this->get_instance_name($instance), $managelink, navigation_node::TYPE_SETTING);
        }
    }

    /**
     * Returns edit icons for the page with list of instances
     * @param stdClass $instance
     * @return array
     */
    public function get_action_icons(stdClass $instance)
    {
        global $OUTPUT;

        if ($instance->enrol !== 'mpay')
        {
            throw new coding_exception('invalid enrol instance!');
        }
        $context = context_course::instance($instance->courseid);
        
        $icons = array();

        if (has_capability('enrol/mpay:config', $context))
        {
            $editlink = new moodle_url("/enrol/mpay/edit.php", array('courseid' => $instance->courseid, 'id' => $instance->id));
            $icons[] = $OUTPUT->action_icon($editlink, new pix_icon('i/edit', get_string('edit'), 'core', array('class' => 'icon')));
        }

        return $icons;
    }

    /**
     * Returns link to page which may be used to add new instance of enrolment plugin in course.
     * @param int $courseid
     * @return moodle_url page url
     */
    public function get_newinstance_link($courseid)
    {

        global $DB;
        $context = context_course::instance($courseid, MUST_EXIST);

        if (!has_capability('moodle/course:enrolconfig', $context) or !has_capability('enrol/mpay:config', $context))
        {
            return NULL;
        }
        if ($DB->get_records('enrol', array('name' => 'mpay', 'courseid' => $courseid)))
        {
            //Only allow on instance
            return NULL;
        }
        else
        {
            return new moodle_url('/enrol/mpay/edit.php', array('courseid' => $courseid));
        }
    }

    /**
     * Creates course enrol form, checks if form submitted
     * and enrols user if necessary. It can also redirect.
     *
     * @param stdClass $instance
     * @return string html text, usually a form in a text box
     */
    function enrol_page_hook(stdClass $instance)
    {
        global $CFG, $USER, $OUTPUT, $PAGE, $DB, $COURSE;

        ob_start();

        if ($DB->record_exists('user_enrolments', array('userid' => $USER->id, 'enrolid' => $instance->id)))
        {
            return ob_get_clean();
        }

        if ($instance->enrolstartdate != 0 && $instance->enrolstartdate > time())
        {
            return ob_get_clean();
        }

        if ($instance->enrolenddate != 0 && $instance->enrolenddate < time())
        {
            return ob_get_clean();
        }

        $course = $DB->get_record('course', array('id' => $instance->courseid));

        $strloginto = get_string("loginto", "", $course->shortname);
        $strcourses = get_string("courses");

        $context = context_course::instance($course->id);
        if ($users = get_users_by_capability($context, 'moodle/course:update', 'u.*', 'u.id ASC', '', '', '', '', false, true))
        {
            $users = sort_by_roleassignment_authority($users, $context);
            $teacher = array_shift($users);
        }
        else
        {
            $teacher = false;
        }
        if (isguestuser())
        { // force login only for guest user, not real users with guest role
            if (empty($CFG->loginhttps))
            {
                $wwwroot = $CFG->wwwroot;
            }
            else
            {
                // This actually is not so secure ;-), 'cause we're
                // in unencrypted connection...
                $wwwroot = str_replace("http://", "https://", $CFG->wwwroot);
            }
            echo '<div class="mdl-align">';
            echo '<p><b></b></p>';
            echo '<p><a href="' . $wwwroot . '/login/">' . get_string('loginsite') . '</a></p>';
            echo '</div>';
        }
        else
        {
            $PAGE->requires->string_for_js('check', 'enrol_mpay');
            $PAGE->requires->string_for_js('select_bank', 'enrol_mpay');
            $PAGE->requires->string_for_js('no_coupon', 'enrol_mpay');
            $PAGE->requires->string_for_js('select_payment', 'enrol_mpay');

            //Default yui sebsoft extenstion
            $module = array('name' => 'sebsoftJs', 'fullpath' => '/enrol/mpay/js/sebsoft.plugin.yui.js', 'requires' => array('transition', 'node'));
            $PAGE->requires->js_module($module);
            $PAGE->requires->js('/enrol/mpay/js/sebsoft.plugin.yui.js', true);
            //
            $module = array('name' => 'mpay', 'fullpath' => '/enrol/mpay/js/yui-custom.js', 'requires' => array('transition', 'node'));
            $PAGE->requires->js_module($module);
            $PAGE->requires->js('/enrol/mpay/js/yui-custom.js', true);

            //
            $id = required_param('id', PARAM_INT);
            $mpayForm = new payForm(PAYMENT_URL, array('id' => $id, 'course' => $COURSE));
            
            $mpayForm->display();
        }
        return $OUTPUT->box(ob_get_clean());
    }

    public function init_xmlrpc_connection($incron=true)
    {
        global $CFG;

        if ($incron)
        {
            mtrace('==> Initialising webservice library');
        }
        # Only require if the class hasn't been setup right, otherwise it may interfere
        # with other products
        if (!class_exists('IXR_Client', false))
        {
            require_once (dirname(__FILE__) . "/class/IXR_Library.inc.php");
        }

        $reqfile = $CFG->dirroot . "/enrol/mpay/version.php";
        require($reqfile);
        $clientversion = $plugin->version;

        if (SYSTEM_SSL == false)
        {
            $ixr = new IXR_Client(SYSTEM_URL, false, 80, false, $clientversion);
        }
        else
        {
            $ixr = new IXR_ClientSSL(SYSTEM_URL, false, 443, false, $clientversion);
        }

        if (!is_a($ixr, 'IXR_Client') && !is_a($ixr, 'IXR_ClientSSL'))
        {
            mtrace("==> Could not instantiate webservice client, please check your PHP configuration - This is CRITICAL");
            return false;
        }

        # Test if the system is ready for use
        if ($incron)
        {
            mtrace('==> Checking if system is ready to be used...');
        }
        # If the server doesn't respond within 10 sec, we assume it's not ready
        $ixr->timeout = 10;
        try
        {
            $ixr->query('proxy.serverready', time());
            $response = $ixr->getResponse();
            if ($response != true)
            {
                if ($incron)
                {
                    mtrace('==> mPay webservice responded in time, but is not ready for requests');
                }
                return false;
            }
            else
            {
                if ($incron)
                {
                    mtrace('==> Ready!');
                }
            }
        }
        catch (Exception $ex)
        {
            if ($incron)
            {
                mtrace("==> CRITICAL error detected @ mPay cron :: " . $ex->getMessage());
            }
            return false;
        }

        if ($incron)
        {
            mtrace('==> System standing by for requests...');
        }
        $ixr->timeout = false;
        $this->ixr = $ixr;
        return $ixr;
    }

    public function cron()
    {
        global $CFG, $DB;
        //Debug::getQuerys();
        # Run cron for mPay...
        mtrace(' ');
        mtrace('Moodle Payments cron starting...');
        /**
         * If the moodle cron is being executed every 5 minutes...
         * This would run 288 times a day for each installation!
         *
         * Way too much, so block running for 80 % of the time...
         * So it's only ran 57 times a day (distributed over 24 hours, so once every half hour or something...
         * Which is more than enough really.
         *
         * Note to self: Up this to < 90 % in next version if this loads the servers too heavy...
         */
        srand((double) microtime() * 10000000);
        $random100 = rand(0, 100);
        if ($random100 < 80 && DEVMODE == false)
        {
            // Approximately 80% of the time.
            mtrace('==> mPay enrolment plugin reported it is not needed to start processing now!');
            mtrace('Terminating cron execution of Moodle Payments.');
            mtrace(' ');
            return true;
        }
        unset($random100);

        # XML-RPC communications for webservice
        $ixr = $this->init_xmlrpc_connection();

        if ($ixr === false)
        {
            mtrace("==> mPay webservice reported system is not ready for use, please try again later");
            return true;
        }

        # If the demo mode has been disabled...
        # First delete all ideal issuers that have 'demo_mode' = 1, issuer simulator kinds sucks in production
        if ($CFG->enrol_mpay_payment_methode_demomode)
        {
            # Warning, we are in demo mode! - We should do something to make sure the admin knows we're in demo mode
        }
        else
        {
            $DB->delete_records('enrol_mpay_idealbank', array('demo_mode' => 1));
        }

        # Phase 1: get available payment methods
        try
        {
            mtrace('==> Refreshing payment methods');
            $ixr->query('system.getavailablepaymentmethods', array('websitecode' => $CFG->enrol_mpay_websitecode));
            $response = $ixr->getResponse();
            if (isset($response['faultCode']))
            {
                mtrace("==> Error:: " . $response['faultString']);
                return false;
            }
            foreach ($response as $name => $paymentmethod)
            {
                $paymentmethod['systemname'] = $name;

                # Get current record with id
                $record = $DB->get_record('enrol_mpay_methods', array('method_id' => $paymentmethod['id']));
                # Ehm, it doesn't exist, I should really add this
                if ($record == false)
                {
                    $obj = new stdClass();

                    # Compensate for the irritating 'unset primairy key' behavior in Moodle
                    $obj->method_id = $paymentmethod['id'];

                    foreach ($paymentmethod as $key => $val)
                    {
                        $obj->$key = $val;
                    }
                    if (!$DB->insert_record('enrol_mpay_methods', $obj))
                    {
                        mtrace('==> [non-critical] Failed to add payment method to db :: ' . $obj->name);
                    }
                }
                else
                {
                    $changed = false;

                    foreach ($record as $key => $val)
                    {
                        if (isset($paymentmethod[$key]))
                        {
                            if ($val != $paymentmethod[$key])
                            {
                                $changed = true;
                            }
                        }
                    }

                    if ($changed)
                    {
                        $obj = new stdClass();
                        foreach ($paymentmethod as $key => $val)
                        {
                            $obj->$key = $val;
                        }
                        # Irritating behavior compensation!
                        $obj->id = $record->id;

                        if (!$DB->update_record('enrol_mpay_methods', $obj))
                        {
                            mtrace('==> [non-critical] Failed to change payment method in db :: ' . $obj->name);
                        }
                        else
                        {
                            mtrace('==> Updated payment method with success');
                        }
                    }
                    else
                    {
                        mtrace("==> No changes in payment method");
                    }
                }
            }
        }
        catch (Exception $ex)
        {
            mtrace("==> CRITICAL error detected @ mPay cron :: " . $ex->getMessage());
            return true;
        }

        # Phase 2: for iDeal, get issuers
        try
        {
            mtrace('==> Refreshing list of iDeal issuers');
            $ixr->query('ideal.getissuers', array('websitecode' => $CFG->enrol_mpay_websitecode));
            $response = $ixr->getResponse();
            foreach ($response as $issuer)
            {
                # Get current record with id
                $record = $DB->get_record('enrol_mpay_idealbank', array('bankid' => $issuer['id']));
                # Ehm, it doesn't exist, I should really add this
                if ($record == false)
                {
                    $obj = new stdClass();

                    # Compensate for the irritating 'unset primairy key' behavior in Moodle
                    $obj->bankid = $issuer['id'];
                    $obj->demo_mode = 0;
                    $obj->imgurl = '';

                    foreach ($issuer as $key => $val)
                    {
                        $obj->$key = $val;
                    }
                    if (!$DB->insert_record('enrol_mpay_idealbank', $obj))
                    {
                        mtrace('==> [non-critical] Failed to add ideal issuer to db :: ' . $obj->name);
                    }
                    else
                    {
                        mtrace('==> Added iDeal issuer ' . $obj->name);
                    }
                }
                else
                {
                    $changed = false;
                    foreach ($record as $key => $val)
                    {
                        if (isset($issuer[$key]))
                        {
                            if ($val != $issuer[$key])
                            {
                                if ($key == 'id')
                                {
                                    continue;
                                }
                                else
                                {
                                    $changed = true;
                                }
                            }
                        }
                    }

                    if ($changed)
                    {
                        $obj = new stdClass();
                        foreach ($issuer as $key => $val)
                        {
                            $obj->$key = $val;
                        }

                        # Irritating behavior compensation!
                        $obj->id = $record->id;

                        if (!$DB->update_record('enrol_mpay_idealbank', $obj))
                        {
                            mtrace('==> [non-critical] Failed to change ideal issuer in db :: ' . $obj->name);
                        }
                        else
                        {
                            mtrace('==> Updated record ' . $obj->name);
                        }
                    }
                    else
                    {
                        mtrace('==> No changes in ' . $issuer['name']);
                    }
                }
            }
        }
        catch (Exception $ex)
        {
            mtrace("==> CRITICAL error detected @ mPay cron :: " . $ex->getMessage());
            return true;
        }

        # Phase 3: get available multilanguage translations
        try
        {
            mtrace('==> Refreshing translations');
            $ixr->query('system.getTranslations', array('websitecode' => $CFG->enrol_mpay_websitecode));
            $response = $ixr->getResponse();
            if (isset($response['faultCode']))
            {
                mtrace("==> Error:: " . $response['faultString']);
                return false;
            }
            # We've got all the translations, so we can safely reset the table now!
            foreach ($response as $translations)
            {
                // mtrace($translations['code']);
                $DB->delete_records('enrol_mpay_idealbank', array('code' => $translations['code'], 'lang' => $translations['lang']));

                $obj = new stdClass();
                foreach ($paymentmethod as $key => $val)
                {
                    $obj->$key = $val;
                }
                if (!$DB->insert_record('enrol_mpay_methods', $obj))
                {
                    mtrace('==> [non-critical] Failed to add payment method to db :: ' . $obj->name);
                }
            }
        }
        catch (Exception $ex)
        {
            mtrace("==> CRITICAL error detected @ mPay cron :: " . $ex->getMessage());
            return true;
        }
        //updating the LOCAL DB
        $currentCoupons = $this->getCoupons(0,false);

        if($currentCoupons)
        {
            foreach($currentCoupons as $coupon)
            {
                $coupon = (object)$coupon;
                $obj            = array();
                $obj['id_mpay']   = $coupon->id;

                $result = $DB->get_record('enrol_mpay_couponcode',$obj);

                //prepare item
                $object = new stdClass();
                $object->courseid = $coupon->courseid;
                $object->validfrom = $coupon->validfrom;
                $object->validto = $coupon->validto;
                $object->percentage = $coupon->percentage;
                $object->code = $coupon->code;
                $object->id_mpay = $coupon->id;

                if(!$result)
                {
                    //insert
                    $DB->insert_record('enrol_mpay_couponcode',$object);
                }
                else
                {
                    //update
                     $object->id =  $result->id;
                     $DB->update_record('enrol_mpay_couponcode',$object);
                }
            }
        }


        # TODO: hmm, it might be interesting to force a check of payment statusses in a next version...
        # However the server should report them as well, so it's not critical for this version.

        unset($ixr);
        mtrace('==> Done with webservices... Local integrity check running');
        #TODO: check if the prices in the courses aren't above the selected for the (primairy) payment method!
        mtrace('Completed run for mPay...');
        mtrace(' ');
    }

    public function getCoupons($id=0, $useDb=false)
    {
        global $CFG, $DB;
        if ($useDb == false)
        {
            $ixr = $this->init_xmlrpc_connection(false);
            try
            {
                if ($id > 0)
                {
                    $ixr->query('discount.getall', array('websitecode' => $CFG->enrol_mpay_websitecode, 'cmd' => 'get', 'id' => $id));
                }
                else
                {
                    $ixr->query('discount.getall', array('websitecode' => $CFG->enrol_mpay_websitecode, 'cmd' => 'get'));
                }
                $response = $ixr->getResponse();
                return $response;
            }
            catch (Exception $ex)
            {
                print_error('Communications issue with Moodle Payments, please try again later - Or contact Avetica for support.');
            }
        }
        else
        {
            if ($results = $DB->get_records('enrol_mpay_couponcode', array(), 'id ASC'))
            {
                $tmpArray = array();
                foreach ($results as $result)
                {
                    $tmpArray[] = (array) $result;
                }
                return $tmpArray;
            }
            else
            {
                return false;
            }
        }
    }

    public function addCoupon($data)
    {
        global $CFG, $DB;
        $ixr = $this->init_xmlrpc_connection(false);
        try
        {
            $request = array();
            $request['websitecode'] = $CFG->enrol_mpay_websitecode;
            $request['cmd'] = 'add';
            $request['percentage'] = floatval($data->percentage);
            $request['newprice'] = 0; // Not supported yet

            $request['course'] = intval($data->courseid);
            $request['courseid'] = intval($data->courseid);

            $request['code'] = $data->code;
            $request['validfrom'] = $data->validfrom;
            $request['validto'] = $data->validto;

            $ixr->query('discount.add', $request);
            $response = $ixr->getResponse();

            if ($response == true)
            {
                //oke succes we save local
                $std = new stdClass();
                $std->id_mpay = $response;
                $std->courseid = $request['course'];
                $std->code = $request['code'];
                $std->percentage = $request['percentage'];
                $std->validto = $request['validto'];
                $std->validfrom = $request['validfrom'];

                $DB->insert_record('enrol_mpay_couponcode', $std);
            }
            return $response;
        }
        catch (Exception $ex)
        {
            //Debug::show($ex);
            print_error('Communications issue with Moodle Payments, please try again later - Or contact Avetica for support.');
        }
    }

    public function editCoupon($data)
    {
        global $CFG, $DB;
        $ixr = $this->init_xmlrpc_connection(false);

        try
        {
            $request = array();
            $request['websitecode'] = $CFG->enrol_mpay_websitecode;
            $request['cmd'] = 'edit';
            $request['percentage'] = floatval($data->percentage);
            $request['newprice'] = 0; // Not supported yet

            $request['courseid'] = intval($data->courseid);
            $request['code'] = $data->code;
            $request['validfrom'] = $data->validfrom;
            $request['validto'] = $data->validto;
            $request['id'] = $data->id;

            $ixr->query('discount.edit', $request);
            $response = $ixr->getResponse();

            if ($response == true)
            {
                $std = new stdClass();
                $item = $DB->get_record('enrol_mpay_couponcode', array('id_mpay' => $data->id));

                if (!empty($item))
                {
                    $std->id = $item->id;
                    $std->courseid = intval($data->courseid);
                    $std->code = $request['code'];
                    $std->percentage = $request['percentage'];
                    $std->validto = $request['validto'];
                    $std->validfrom = $request['validfrom'];
                    $std->id_mpay = $request['id'];
                    $DB->update_record('enrol_mpay_couponcode', $std);
                }
            }
            else
            {

                throw new Exception('Not a valid response', '500');
            }

            return $response;
        }
        catch (Exception $ex)
        {
            //Debug::show($ex);
            print_error('Communications issue with Moodle Payments, please try again later - Or contact Avetica for support.');
        }
    }

    public function deleteCoupon($iId)
    {
        global $CFG, $DB;
        $ixr = $this->init_xmlrpc_connection(false);
        try
        {
            $request = array();
            $request['websitecode'] = $CFG->enrol_mpay_websitecode;
            $request['cmd'] = 'delete';
            $request['id'] = $iId;

            $DB->delete_records('enrol_mpay_couponcode', array('id_mpay' => $iId));
            $ixr->query('discount.delete', $request);
            $response = $ixr->getResponse();
            return $response;
        }
        catch (Exception $ex)
        {
            //Debug::show($ex);
            print_error('Communications issue with Moodle Payments, please try again later - Or contact Avetica for support.');
        }
    }

    public function getVats($id=0)
    {
        global $CFG, $DB;
        $ixr = $this->init_xmlrpc_connection(false);
        try
        {
            if ($id > 0)
            {
                $ixr->query('vat.getall', array('websitecode' => $CFG->enrol_mpay_websitecode, 'cmd' => 'get', 'id' => $id));
            }
            else
            {
                $ixr->query('vat.getall', array('websitecode' => $CFG->enrol_mpay_websitecode, 'cmd' => 'get'));
            }
            $response = $ixr->getResponse();
            return $response;
        }
        catch (Exception $ex)
        {
            print_error('Communications issue with Moodle Payments, please try again later - Or contact Avetica for support.');
        }
    }

    public function addVat($data)
    {
        global $CFG, $DB;
        $ixr = $this->init_xmlrpc_connection(false);
        try
        {
            $request = array();
            $request['websitecode'] = $CFG->enrol_mpay_websitecode;
            $request['cmd'] = 'add';
            $request['percentage'] = floatval($data->percentage);
            $request['course'] = intval($data->course);

            $ixr->query('vat.add', $request);
            $response = $ixr->getResponse();

            if (is_numeric($response))
            {
                $std = new stdClass();
                $std->courseid = $request['course'];
                $std->percentage = $request['percentage'];

                $DB->insert_record('enrol_mpay_vat', $std);
            }
            return $response;
        }
        catch (Exception $ex)
        {
            print_error('Communications issue with Moodle Payments, please try again later - Or contact Avetica for support.');
        }
    }

    public function editVat($data)
    {
        global $CFG, $DB;
        $ixr = $this->init_xmlrpc_connection(false);
        try
        {
            $request = array();
            $request['websitecode'] = $CFG->enrol_mpay_websitecode;
            $request['cmd'] = 'edit';
            $request['percentage'] = floatval($data->percentage);
            $request['course'] = intval($data->course);
            $request['id'] = $data->id;

            $ixr->query('vat.edit', $request);
            $response = $ixr->getResponse();

            if (is_numeric($response))
            {
                $std = new stdClass();
                $std->courseid = $request['course'];
                $std->percentage = $request['percentage'];
                $item = $DB->get_record('enrol_mpay_vat', array('courseid' => $request['course']));
                $std->id = $item->id;
                $DB->update_record('enrol_mpay_vat', $std);
            }
            return $response;
        }
        catch (Exception $ex)
        {
            print_error('Communications issue with Moodle Payments, please try again later - Or contact Avetica for support.');
        }
    }

    public function deleteVat($iId)
    {
        global $CFG, $DB;
        $ixr = $this->init_xmlrpc_connection(false);
        try
        {
            $request = array();
            $request['websitecode'] = $CFG->enrol_mpay_websitecode;
            $request['cmd'] = 'delete';
            $request['id'] = $iId;

            $ixr->query('vat.delete', $request);
            $response = $ixr->getResponse();

            if (is_numeric($response))
            {
                $vat = self::getVats($iId);
                $DB->delete_records('enrol_mpay_vat', array('courseid' => $vat['courseid']));
            }

            return $response;
        }
        catch (Exception $ex)
        {
            print_error('Communications issue with Moodle Payments, please try again later - Or contact Avetica for support.');
        }
    }

}